# -*- coding:utf-8 -*-
# @Author    : g1879
# @date      : 2020-07-30
# @email     : g1879@qq.com
# @File      : list_page.py
"""
ListPage类是翻页式列表页基类，继承自DrissionPage的MixPage类。
专门用于处理翻页式列表页面。如商城产品页、文章列表页。
它抽取了翻页列表共有的特征，即栏目名、下一页按钮、总页数、数据行、列，
封装了对页面的基本读取和操作方法，可适应几乎所有翻页列表。

它封装了以下方法：
- 读取读取一定范围页数据列表
- 获取当前页数据列表
- 获取当前页行元素对象列表
- 跳转到下一页
- 跳转到特定页
- 获取总页数

ListPage类派生了两个子类，实际使用时根据不同情况选用：
- ListPageD  - 用于处理由js加载的列表页。基于MixPage的d模式，即使用selenium。
- ListPageS  - 用于处理无须js加载的列表页。基于MixPage的s模式，即使用requests。

返回列表格式：
    [
        ['参数1结果', '参数2结果', ...],
        ['参数1结果', '参数2结果', ...],
        ...
    ]

使用时，根据实际情况派生之类，重写to_next_page()方法。
若页面有跳转功能，建议重写to_any_page()方法。
"""
from time import sleep
from typing import Union
import re

from DrissionPage import Drission

from .recorder import Recorder
from .targets import Targets
from .base_page import BasePage
from .paths import Paths


class ListPage(BasePage):
    """列表页基类                                                    \n
    列表页基类提取了列表页面共有的特征，封装了对页面的基本读取和操作方法。
    """

    def __init__(self,
                 paths: Union[Paths, dict],
                 index_url: str = None,
                 mode: str = 's',
                 timeout: float = 10,
                 drission: Drission = None):
        """初始化函数                                       \n
        :param paths: 页面元素管理对象
        :param index_url: 列表第一页url
        :param mode: 's' 或 'd'，MixPage模式
        :param timeout: 超时时间
        :param drission: Drission对象
        """
        super().__init__(paths, index_url, mode, timeout, drission)
        self._pages_count = self._get_pages_count_from_paths()  # 列表总页数
        self._num_param = None  # url 中页码参数
        self.step = 1  # 翻页编码步长
        self.first_num = 1  # 第一页页码，0 或 1

    @property
    def current_page_num(self) -> int:
        """返回当前页码"""
        if self.num_param:
            self._current_page_num = self._get_page_num_from_url()

        return self._current_page_num

    @property
    def pages_count(self) -> Union[int, None]:
        """返回总页数"""
        return self._pages_count

    @pages_count.setter
    def pages_count(self, num: int) -> None:
        """手动设置总页数           \n
        :param num: 总页数
        :return: None
        """
        self._pages_count = num

    @property
    def num_param(self) -> str:
        """返回 url 中页码参数"""
        return self._num_param

    @num_param.setter
    def num_param(self, param: str) -> None:
        """设置 url 中页码参数            \n
        :param param: url 中页码参数
        :return: None
        """
        if not param.startswith('/') and f'{param}=' not in self.index_url:
            raise ValueError('要使用页码翻页，须在index_url中包含该参数')

        self._num_param = param

    def get(self,
            url: str,
            go_anyway=False,
            show_errmsg: bool = False,
            retry: int = 2,
            interval: float = 1,
            **kwargs) -> Union[bool, None]:
        """跳转到一个url                                         \n
        :param url: 目标url
        :param go_anyway: 若目标url与当前url一致，是否强制跳转
        :param show_errmsg: 是否显示和抛出异常
        :param retry: 重试次数
        :param interval: 重试间隔（秒）
        :param kwargs: 连接参数，s模式专用
        :return: url是否可用
        """
        self._pages_count = self._get_pages_count_from_paths()  # 列表总页数
        self._current_page_num = 1
        return super().get(url, go_anyway, show_errmsg, retry, interval, **kwargs)

    def get_list(self,
                 targets: Union[Targets, list, tuple],
                 begin_page: int = None,
                 count: int = None,
                 stop_when_empty: bool = True,
                 wait: float = None,
                 show_msg: bool = True,
                 recorder: Recorder = None,
                 return_data: bool = True) -> list:
        """根据targets的内容爬取列表内容            \n
        :param targets: 要爬取的内容
        :param begin_page: 起始页码
        :param count: 爬取页面数量
        :param stop_when_empty: 空页是否停止
        :param wait: 翻页后等待几秒
        :param show_msg: 是否显示爬取信息
        :param recorder: 记录器对象
        :param return_data: 是否返回数据
        :return: 结果列表
        """
        if begin_page:
            self.to_page(begin_page)
        else:
            begin_page = self.current_page_num

        if not stop_when_empty:
            if not self.pages_count and not count:
                raise KeyError('须传入爬取页数')

            if count and (not isinstance(count, int) or count < 1):
                raise TypeError('count须传入正整数')

        # 获取要爬页数
        if self.pages_count and (not count or count > self.pages_count - begin_page + 1):
            count = self.pages_count - begin_page + 1

        data_list = []
        got_page = 0

        while True:
            if show_msg:
                print(f'\n第{begin_page + got_page}页')
                print(self.url)

            current_list = self.get_current_list(targets, show_msg=show_msg)

            if not current_list and stop_when_empty:
                if show_msg:
                    print('空列表，终止。')
                break

            if return_data:
                data_list.extend(current_list)

            if recorder:
                recorder.add_data(current_list)

            got_page += 1

            if count and count == got_page:
                break

            if not self.to_next_page(wait):
                break

        if recorder:
            recorder.record()

        return data_list

    def to_page(self, num: int, wait: float = None) -> None:
        """跳转到任意页                        \n
        :param num: 页码
        :param wait: 翻页后等待秒数
        :return: None
        """
        if self.num_param and self._to_page_by_num(num) is not False:
            self._current_page_num = num
            return
        else:

            if self.pages_count and not 0 < num <= self.pages_count:
                raise KeyError('始页应该在总页数范围内')

            if self.index_url:
                self.to_first_page()

                for _ in range(num - 1):
                    self.to_next_page(wait=wait)
            else:
                raise ValueError('首页url未设置')

    def to_next_page(self, wait: float = None) -> bool:
        """跳转到下一页                        \n
        :param wait: 跳转后等待秒数
        :return: 跳转是否成功
        """
        if self.num_param:
            current_num = self._get_page_num_from_url()
            is_ok = self._to_page_by_num(current_num + 1)

        else:
            next_btn = self.ele(f'{self.paths.type}:{self.paths.next_btn}', timeout=2)

            if next_btn:
                if self.mode == 's':
                    is_ok = self._to_next_s_mode()
                else:
                    is_ok = self._to_next_d_mode()
            else:
                is_ok = False

        if is_ok is not False:
            self._current_page_num += 1
            result = True
        else:
            result = False

        if wait:
            sleep(wait)

        return result

    def _to_next_s_mode(self) -> bool:
        """s模式下跳转到下一页的方法                       \n
        有必要时重载这个方法处理特殊的跳转规则
        """
        next_btn = self.ele(f'{self.paths.type}:{self.paths.next_btn}')

        if next_btn:
            return self.get(next_btn.attr('href'), show_errmsg=True)
        else:
            return False

    def _to_next_d_mode(self) -> bool:
        """d模式下跳转到下一页的方法                       \n
        有必要时重载这个方法处理特殊的跳转规则
        """
        next_btn = self.ele(f'{self.paths.type}:{self.paths.next_btn}', timeout=2)

        if next_btn:
            next_btn.click(by_js=True)
            sleep(.5)
            return True
        else:
            return False

    def _to_page_by_num(self, num: int, wait: float = None) -> Union[bool, None]:
        """根据页码跳转                        \n
        :param num: 页码
        :param wait: 跳转后等待秒数
        :return: 跳转后是否成功
        """
        if wait:
            sleep(wait)

        if not self.num_param:
            return False

        if self.num_param.startswith('/'):
            new_url = re.sub(f'{self.num_param}(\\d*)', f'{self.num_param}{self._page_num_to_url_num(num)}', self.url)
        else:
            new_url = re.sub(f'{self.num_param}=(\\d*)', f'{self.num_param}={self._page_num_to_url_num(num)}', self.url)

        return self.get(new_url, go_anyway=True, show_errmsg=True, retry=6, interval=10)

    def _get_page_num_from_url(self) -> Union[int, None]:
        """从url中获取页码"""
        if not self.num_param:
            return None

        if self.num_param.startswith('/'):
            r = re.search(f'{self.num_param}(\\d*)', self.url)
        else:
            r = re.search(f'{self.num_param}=(\\d*)', self.url)

        if r is None:
            return None

        num = int(r.group(1)) if r else None
        num = num // self.step + (1 - self.first_num)
        return num

    def _page_num_to_url_num(self, num: int) -> int:
        """把页码转换成url的数字                    \n
        :param num: 第几页
        :return: url中页码数字
        """
        return (num - 1 + self.first_num) * self.step

    def _get_pages_count_from_paths(self) -> Union[int, None]:
        """获取总页数"""
        if not self.paths.pages_count:
            return None

        total = self._get_value(self, self.paths.pages_count)

        return int(total) if total else 1



# -*- coding:utf-8 -*-
# @Author    : g1879
# @date      : 2020-11-17
# @email     : g1879@qq.com
# @File      : paths.py

from typing import Union


class Paths(object):
    """路径管理对象                        \n
    用于管理列表页面中关键元素的路径信息"""
    __attrs__ = ['rows', 'cols', 'page_count', 'next_btn', 'more_btn', 'container']

    def __init__(self, path_type: str = None, paths_dict: dict = None):
        """初始化                                           \n
        :param path_type: 路径的类型， 'css' 或 'xpath'
        :param paths_dict: 包含路径信息的字典
        """
        self.type = path_type

        self._rows = None
        self._cols = {}
        self._next_btn = None
        self._pages_count = None
        self._more_btn = None
        self._container = None

        if paths_dict:
            self.from_dict(paths_dict)

    def __call__(self) -> dict:
        """调用时把路径信息以字典格式返回"""
        return self.as_dict()

    @property
    def type(self) -> str:
        """返回路径的类型，css或xpath"""
        if self._type is None:
            raise ValueError('路径类型未指定。')

        return self._type

    @type.setter
    def type(self, paths_type: str) -> None:
        """设置路径类型                        \n
        :param paths_type: 'css' 或 'xpath'
        :return: None
        """
        if paths_type not in ('css', 'xpath', None):
            raise ValueError("type参数只能为 'css' 或 'xpath'。")

        self._type = paths_type

    @property
    def rows(self) -> str:
        """返回行元素的路径"""
        return self._rows

    @rows.setter
    def rows(self, path: str) -> None:
        """设置行元素的路径         \n
        :param path: 行元素的路径
        :return: None
        """
        if not isinstance(path, str):
            raise TypeError('path参数只能是str。')

        self._rows = path

    @property
    def next_btn(self) -> str:
        """返回下一页按钮的路径"""
        return self._next_btn

    @next_btn.setter
    def next_btn(self, path: str) -> None:
        """设置下一页按钮的路径         \n
        :param path: 下一页按钮的路径
        :return: None
        """
        if not isinstance(path, str):
            raise TypeError('path参数只能是str。')

        self._next_btn = path

    @property
    def pages_count(self) -> Union[str, list, tuple]:
        """返回总页数元素的定位符列表"""
        return self._pages_count

    @pages_count.setter
    def pages_count(self, path_or_loc: Union[str, list, tuple]) -> None:
        """设置总页数元素的定位符                  \n
        :param path_or_loc: 总页数元素的定位符
        :return: None
        """
        if isinstance(path_or_loc, (list, tuple)) and not 0 < len(path_or_loc) < 4:
            raise ValueError('定位符格式不正确。应为str，或长度小于4的list或tuple。')

        self._pages_count = path_or_loc

    @property
    def more_btn(self) -> str:
        """返回加载更多按钮元素的路径"""
        return self._more_btn

    @more_btn.setter
    def more_btn(self, path: str) -> None:
        """设置加载更多按钮元素的路径              \n
        :param path:加载更多按钮元素的路径
        :return: None
        """
        if not isinstance(path, str):
            raise TypeError('path参数只能是str。')

        self._more_btn = path

    @property
    def container(self) -> str:
        """返回滚动列表页的容器路径"""
        return self._container

    @container.setter
    def container(self, path: str) -> None:
        """设置滚动列表页的容器路径    \n
        :param path: 容器的路径
        :return: None
        """
        if not isinstance(path, str):
            raise TypeError('path参数只能是str。')

        self._container = path

    @property
    def cols(self) -> dict:
        """返回保存的列路径"""
        return self._cols

    def get_col(self, col_name: str) -> str:
        """返回某列元素的路径                             \n
        :param col_name: 列名
        :return: 列的路径
        """
        return self._cols[col_name] if col_name in self._cols else None

    def set_cols(self, cols_data: Union[dict, list, tuple]) -> None:
        """批量设置列路径                               \n
        :param cols_data: list或tuple格式保存的列信息
        :return: None
        """
        if isinstance(cols_data, dict):
            self._cols = cols_data

        elif isinstance(cols_data, (list, tuple)):
            # 一维数组，即 (key, paths_or_dict)
            if len(cols_data) == 2 and isinstance(cols_data[0], str) and isinstance(cols_data[1], str):
                self.set_col(cols_data[0], cols_data[1])

            # 二维数组，即 ((key, paths_or_dict), (key, paths_or_dict), ...)
            else:
                for i in cols_data:
                    if (not isinstance(i, (list, tuple)) or len(i) != 2
                            or not isinstance(i[0], str) or not isinstance(i[1], str)):
                        raise TypeError

                    self.set_col(i[0], i[1])

        else:
            raise TypeError('参数cols类型错误。')

    def set_col(self, col_name: str, path: str) -> None:
        """设置一列的路径                  \n
        :param col_name: 列名
        :param path: 列的路径
        :return: None
        """
        self._cols[col_name] = path

    def from_dict(self, paths_dict: dict):
        """从字典中读取并设置路径信息                  \n
        :param paths_dict: 以字典形式保存的列信息
        :return: 返回自己
        """
        self.type = get_paths_type_from_dict(paths_dict)

        for argument in paths_dict:
            if argument in self.__attrs__:
                if argument == 'cols':
                    self.set_cols(paths_dict['cols'])
                else:
                    setattr(self, argument, paths_dict[argument])

        return self

    def as_dict(self) -> dict:
        """把路径信息以字典格式返回"""
        return_dict = {key: getattr(self, key) for key in self.__attrs__}
        return_dict['type'] = self.type
        return return_dict


class Xpaths(Paths):
    def __init__(self, paths_dict: dict = None):
        """类型为xpath的路径类                      \n
        :param paths_dict: 以字典形式保存的路径信息
        """
        super().__init__(paths_dict=paths_dict)
        self._type = 'xpath'

    @property
    def type(self) -> str:
        """返回路径类型"""
        return super().type

    @type.setter
    def type(self, path_type: str) -> None:
        """重载父类属性，不允许改变该值                       \n
        :param path_type: 路径类型，'css'或'xpath'
        :return: None
        """
        pass


class CssPaths(Paths):
    def __init__(self, paths_dict: dict = None):
        """类型为css selector的路径类                      \n
        :param paths_dict: 以字典形式保存的路径信息
        """
        super().__init__(paths_dict=paths_dict)
        self._type = 'css'

    @property
    def type(self) -> str:
        """返回路径类型"""
        return super().type

    @type.setter
    def type(self, path_type: str) -> None:
        """重载父类属性，不允许改变该值                   \n
        :param path_type: 路径类型，'css'或'xpath'
        :return: None
        """
        pass


def get_paths_type_from_dict(paths_dict: dict) -> str:
    """从路径字典中获取路径类型                      \n
    :param paths_dict: 以字典形式保存的路径信息
    :return: 'xpath' 或 'css'
    """
    p_type = paths_dict.get('type', None)

    if not p_type:
        p_type = 'css'
        for i in paths_dict:
            if '/' in paths_dict[i]:
                p_type = 'xpath'
                break

    return p_type



# -*- coding:utf-8 -*-
# @Author    : g1879
# @date      : 2020-11-17
# @email     : g1879@qq.com
# @File      : base_page.py

import re
from abc import abstractmethod
from typing import Union, List

from DrissionPage import MixPage, Drission
from DrissionPage.common import DrissionElement

from .common import format_loc
from .targets import Targets
from .paths import Paths


class BasePage(MixPage):

    def __init__(self,
                 paths: Union[Paths, dict],
                 index_url: str = None,
                 mode: str = 's',
                 timeout: float = 10,
                 drission: Drission = None):
        """列表页面类的基类                        \n
        :param paths: 页面元素管理对象
        :param index_url: 列表第一页url
        :param mode: 's' 或 'd'，MixPage模式
        :param timeout: 超时时间
        :param drission: Drission对象
        """
        super().__init__(drission, mode, timeout)
        self._set_paths(paths)
        self.index_url = index_url
        self._current_page_num = 1

        if index_url:
            self.to_first_page()

    @property
    def paths(self) -> Paths:
        """返回页面路径管理对象"""
        return self._paths

    def _set_paths(self, dict_or_Paths: Union[dict, Paths]) -> None:
        """设置路径对象                                 \n
        :param dict_or_Paths: Paths对象或dict
        :return: None
        """
        if isinstance(dict_or_Paths, Paths):
            self._paths = dict_or_Paths

        elif isinstance(dict_or_Paths, dict):
            self._paths = Paths(paths_dict=dict_or_Paths)

        else:
            raise ValueError('paths格式错误')

    def to_first_page(self) -> None:
        """跳转到首页"""
        if self.index_url:
            self.get(self.index_url, go_anyway=True, show_errmsg=True, retry=6, interval=10)
            self._current_page_num = 1
        else:
            raise ValueError('首页url未设置')

    def get_current_rows(self) -> List[DrissionElement]:
        """返回当前页行对象"""
        return self.eles(f'{self.paths.type}:{self.paths.rows}')

    def get_current_list(self,
                         targets: Union[Targets, list, tuple],
                         show_msg: bool = True) -> List[List[str]]:
        """返回当前页结果列表                    \n
        :param targets: Targets对象
        :param show_msg: 是否显示获取到的内容
        :return: 结果列表
        """
        rows = self.get_current_rows()
        return self._get_list(rows, targets, show_msg)

    @abstractmethod
    def get_list(self,
                 targets: Union[Targets, list, tuple],
                 show_msg: bool = True,
                 wait: float = 0) -> List[List[str]]:
        """返回指定范围的结果列表，由子类实现                    \n
        :param targets: Targets对象
        :param show_msg: 是否实时显示获取到的内容
        :param wait: 每页之间等待时间
        :return: 结果列表
        """
        pass

    @abstractmethod
    def to_next_page(self, wait: float = None) -> bool:
        """跳转到下一页"                    \n
        :param wait: 跳转前的等待时间
        :return: 是否跳转成功
        """
        pass

    def _get_list(self,
                  rows: list,
                  targets: Union[Targets, list, tuple],
                  show_msg: bool = True) -> List[List[str]]:
        """从行对象列表中获取须要的属性值                    \n
        :param rows: 行对象组成的列表
        :param targets: Targets对象
        :param show_msg: 是否实时显示获取到的信息
        :return: 结果列表
        """
        if isinstance(targets, Targets):
            pass
        elif isinstance(targets, (list, tuple)):
            targets = Targets(self.paths, targets)
        else:
            raise TypeError('targets参数只能是Targets、list或tuple。')

        start, stop = targets.start_stop_row

        results_list = []
        for row in rows[start: stop]:
            row_result = [self._get_value(row, target) for target in targets.targets]

            if show_msg:
                print(row_result)

            results_list.append(row_result)

        return results_list

    def _get_value(self,
                   ele_or_page: Union[MixPage, DrissionElement],
                   loc: Union[str, list, tuple]) -> Union[str, None]:
        """根据定位从元素中获取值                                    \n
        :param ele_or_page: 获取值的目标对象
        :param loc: 定位符：(xpath或css路径, [属性名, 正则])
        :return: 获取到的属性值
        """
        xpath_str, attr_str, re_str = format_loc(loc)

        try:
            data = ele_or_page.ele(f'{self.paths.type}:{xpath_str}').attr(attr_str)
            r = re.search(re_str, data, flags=re.DOTALL)
            return r.group(1)

        except AttributeError:
            return None



# -*- coding:utf-8 -*-
# @Author    : g1879
# @date      : 2020-11-17
# @email     : g1879@qq.com
# @File      : targets.py

from typing import Union

from .common import format_loc
from .paths import Paths


class Targets(object):
    """此类用于管理待爬取目标                                           \n
    待爬目标由一个3位的元组表示：(列名, [属性名, 正则表达式])              \n
    如：('作者', 'text', '(.*)')
    其中后两位可省略，当省略时默认获取元素text值
    即传入'作者' 与传入 ('作者', 'text', '(.*)')是等价的
    示例：
        - '作者'
        - '链接', 'href'
        - '页数', 'text', '(\\d+)'
    """

    def __init__(self,
                 paths: Union[Paths, dict],
                 targets: Union[list, tuple] = None):
        """初始化                              \n
        :param paths: Paths对象
        :param targets: 要爬取的内容集合
        """
        if isinstance(paths, Paths):
            self._paths = paths

        elif isinstance(paths, dict):
            self._paths = Paths(paths_dict=paths)

        else:
            raise TypeError('paths参数只能是Paths或dict类型。')

        self._targets = []
        self._start_stop_row = (None, None)

        if targets:
            self.set_targets(targets)

    @property
    def paths(self) -> Paths:
        """返回页面路径管理对象"""
        return self._paths

    @property
    def start_stop_row(self) -> tuple:
        """返回起止行号"""
        return self._start_stop_row

    @start_stop_row.setter
    def start_stop_row(self, start_stop: Union[list, tuple]):
        """设置起止行号"""
        if not isinstance(start_stop, (list, tuple)) or not 0 < len(start_stop) < 3:
            raise ValueError

        self._start_stop_row = (start_stop[0], None) if len(start_stop) == 1 else start_stop

    @property
    def targets(self) -> list:
        """返回所有目标"""
        return self._targets

    @targets.setter
    def targets(self, targets: Union[list, tuple]) -> None:
        """批量设置目标                            \n
        :param targets: 列表或元组格式的目标信息
        :return: None
        """
        self.set_targets(targets)

    def set_targets(self, targets: Union[list, tuple]) -> None:
        """批量设置目标                            \n
        :param targets: 列表或元组格式的目标信息
        :return: None
        """
        if not isinstance(targets, (tuple, list)):
            raise TypeError(f'请传入tuple或list，不是{type(targets)}')

        # 当第一个元素是范围设置时
        if isinstance(targets[0], (tuple, list)) and isinstance(targets[0][0], (int, type(None))):
            if not 0 < len(targets) < 3:
                raise ValueError

            start = 1
            self._start_stop_row = (targets[0][0], None) if len(targets[0]) == 1 else targets[0]

        else:
            start = 0
            self._start_stop_row = (None, None)

        for i in targets[start:]:
            xpath_str, attr_str, re_str = format_loc(i)
            self.add_target(xpath_str, attr_str, re_str)

    def add_target(self, col: str, attr: str = None, re_str: str = None) -> None:
        """添加待爬目标                                     \n
        :param col: 对象的路径，xpath 或 css selector
        :param attr: 要抓取的熟悉
        :param re_str: 提取属性内容的正则表达式
        :return: None
        """
        path_str = self.paths.cols[col]
        attr = attr or 'text'
        re_str = re_str or '(.*)'

        self._targets.append((path_str, attr, re_str))



# -*- coding:utf-8 -*-
from .list_page import ListPage
from .scrolling_page import ScrollingPage
from .paths import Xpaths, CssPaths, Paths
from .targets import Targets
from .recorder import Recorder



#!/usr/bin/env python
# -*- coding:utf-8 -*-

import csv
from pathlib import Path
from typing import Union
import openpyxl


class Recorder(object):
    def __init__(self, file_path: str = None, cache: int = 50):
        """初始化                                                    \n
        :param file_path: 保存的文件路径
        :param cache: 每接收多少个写入文件
        """
        self.cache = cache
        self.file_path = file_path
        self._data = []
        self._before = []
        self._after = []

    def __del__(self) -> None:
        """对象关闭时把剩下的数据写入文件"""
        if self._data:
            self.record()

    def add_data(self, data: Union[list, tuple]) -> None:
        """添加数据                                                           \n
        :param data: 插入的数据，元组或列表
        :return: None
        """
        self._data.extend(data)
        if len(self._data) >= self.cache:
            self.record()

    def record(self) -> None:
        """记录数据"""
        if self._data:
            if self.file_path.endswith('.xlsx'):
                self._to_excel()
            else:
                with open(self.file_path, 'a+', newline='') as f:
                    csv_write = csv.writer(f)
                    for i in self._data:
                        csv_write.writerow(self._before + i + self._after)

            self._data = []

    def _to_excel(self) -> None:
        """把数据记录到excel"""
        if Path(self.file_path).exists():
            wb = openpyxl.load_workbook(self.file_path)
        else:
            wb = openpyxl.Workbook()

        ws = wb.active
        for i in self._data:
            ws.append(self._before + i + self._after)

        wb.save(self.file_path)
        wb.close()

    def set_before(self, before: Union[list, tuple, str]) -> None:
        """在数据前面补充的列                                                    \n
        :param before: 列表、元组或字符串，为字符串时则补充一列
        :return: None
        """
        self.record()

        if not before:
            self._before = []
        elif isinstance(before, (list, tuple)):
            self._before = list(before)
        elif isinstance(before, str):
            self._before = [before]
        else:
            raise TypeError('只能接收list、tuple、str或空。')

    def set_after(self, after: Union[list, tuple, str]) -> None:
        """在数据后面补充的列                                                      \n
        :param after: 列表、元组或字符串，为字符串时则补充一列
        :return: None
        """
        self.record()

        if not after:
            self._after = []
        elif isinstance(after, (list, tuple)):
            self._after = list(after)
        elif isinstance(after, str):
            self._after = [after]
        else:
            raise TypeError('只能接收list、tuple、str或空。')

    def add_head(self, head: Union[list, tuple]) -> None:
        """设置表头                                                    \n
        :param head: 表头，列表或元组
        :return: None
        """
        if self.file_path.endswith('.xlsx'):
            self._set_excel_head(head)
        else:
            with open(self.file_path, 'w+', newline='') as f:
                content = f.read()
                f.seek(0, 0)
                text = ','.join(head)
                f.write(f'{text}\n{content}')

    def _set_excel_head(self, head: Union[list, tuple]) -> None:
        """设置excel表头                                                    \n
        :param head: 表头，列表或元组
        :return: None
        """
        if Path(self.file_path).exists():
            wb = openpyxl.load_workbook(self.file_path)
        else:
            wb = openpyxl.Workbook()

        ws = wb.active
        ws.insert_rows(1)

        for key, i in enumerate(head, 1):
            ws.cell(1, key).value = i

        wb.save(self.file_path)
        wb.close()
